package functions;
# See man 1 perlmod for the perl module template used here

use strict;
use POSIX qw/ :termios_h strftime /;
use File::Temp qw/ tempfile tempdir /;
use File::Basename qw/basename dirname/;
use warnings;
# Perl mySQL API
use log;

# module template pretext:  START
# 
BEGIN {
    use Exporter   ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # Here: RCS; the q$...$ from RCS act as a quote here(!)
    $VERSION = sprintf "%d.%02d", q$Revision: 1.1 $ =~ /(\d+)/g;

    @ISA         = qw(Exporter);

    # What is exported by default upon the simple "use THISMODULE"
    # (See: camel book, p. 288)
    # It's good manners to have only functions here, no other variables 
    # (putting variables into @EXPORT_OK instead)
    @EXPORT      = qw(  
			&is_bench
			&bench_func
			&get_log_dir
			&is_ltp
			&scp
			&mail
			&process_dirname
			&remove_duplicite_keyvals
			&set_product_release
			$batchmode
		);
    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],

    # What can be additionally exported, if it's specifically required
    # by means of a "use THISMODULE qw/list_of_requested_exports/"
    # (See: camel book, p. 288)
    # In particular, all exported package global _variables_ should go here.
    # _Note_: exporting non-function variables is dangerous and should
    #         be carried out carefully and with restraint.
    @EXPORT_OK   = qw(
		);
}
our @EXPORT_OK;

our $batchmode=0;
my %benchmarks=(
	'dbench[-_]\w[\w\d]*' => 'parse_dbench',
	'bonnie-default' => 'parse_bonnie',
	'tiobench' => 'parse_tiobench',
	'siege_defaultrun|qa_siege_defaultrun.sh|qa_siege' => 'parse_siege',
	'libmicro-bench' => 'parse_libmicro',
	'specweb' => 'parse_specweb',
	'sysbench-\w+' => 'parse_sysbench',
	'interbench' => 'parse_interbench',
	'reaim(?:-(?:compute|dbase|fserver|shared))?' => 'parse_reaim',
	'aim7-(?:compute|dbase|fserver|shared)' => 'parse_aim7',
	'lmbench' => 'parse_lmbench',
	'tiobench[-\w]*' => 'parse_tiobench',
	'kernbench' => 'parse_kernbench'
);



sub is_bench	# 
{
	my $exp='^('.join('|',keys %benchmarks).')';
	return $1 if $_[0] =~ $exp;
	return undef;
}

sub bench_func	# testcase
{
	foreach my $exp( keys %benchmarks )
	{	return $benchmarks{$exp} if $_[0]=~ ('^'.$exp.'$');	}
	return undef;
}


sub get_log_dir	# directory basename
{
	my $logdir;
	my %args=(
		'arch'=>`uname -m`,
		'host'=>`uname -n`,
		'kernel'=>`uname -r`,
		'testsuite'=>'',
		'date'=>'',
		'md5sum'=>'',
		'branch'=>'',
		'product'=>'',
		'release'=>'',
		@_
	);
	my $ltp=&is_ltp( $args{'testsuite'} );
	if( $args{'type'} =~ /^patch/ )
	{
		if( $args{'testsuite'} =~ /^(reaim|newburn)/ )
		{	$logdir='Maintenance/Kernel/'.$args{'product'}.'/'.$args{'arch'}.'/'.$args{'host'}.'/'.$args{'kernel'}.'/'.$args{'testsuite'};	}
		elsif( $ltp )
		{	$logdir='Maintenance/Kernel/'.$args{'product'}.'/'.$args{'arch'}.'/'.$args{'host'}.'/'.$args{'kernel'}.'/ltp/'.$args{'testsuite'}.'-'.$args{'date'};	}
		else
		{	$logdir='Maintenance/Other/'.$args{'md5sum'}.'/'.$args{'product'}.'-'.$args{'arch'}.'/'.$args{'testsuite'}.'-'.$args{'date'};	}
	}
	elsif( $args{'type'} =~ /^kotd/ )
	{
		if( $ltp )
		{	$logdir='kotd/'.$args{'branch'}.'/'.$args{'arch'}.'/'.$args{'host'}.'/'.$args{'kernel'}.'/ltp/'.$args{'testsuite'}.'-'.$args{'date'};	}
		else
		{	$logdir='kotd/'.$args{'branch'}.'/'.$args{'arch'}.'/'.$args{'host'}.'/'.$args{'kernel'}.'/'.$args{'testsuite'};	}
	}
#	elsif( $args{'type'} =~ /^regression/ )
#	{	$logdir=$args{'arch'}.'/'.$args{'host'}.'/'.$args{'kernel'}.'/'.$args{'testsuite'}.'-'.$args{'date'};	}
	else
	{
		$logdir='ProductTests/'.$args{'product'}.'/'.$args{'release'}.'/'.$args{'arch'}.'/'.$args{'host'};
	}
	return $logdir;
}

sub is_ltp # testcase
{	return $_[0] =~ /^(dio|fs|fsx|ipc|ltp-.+|math|mm|openposix|pty|qa_apparmor-ltp|qa_bind|qa_bzip2|qa_clamav|qa_net-snmp|qa_php4|qa_php4-server|qa_samba|quickhit|sched|sched-ltp-realtime|syscalls|timer-create-xx|trivial)(-\d{1,4})*$/;	}

sub scp	# srcdir, destdir
{
	my( $srcdir, $destdir)=@_;
	$destdir="/suse/rd-qa/Export/Results/$destdir";
	my $ret1=system("ssh rd-qa\@qadb.suse.de mkdir -p $destdir");
	&log(LOG_ERR,"Creating remote log directory '$destdir' failed with $ret1") if $ret1;
	my $ret2=system("scp -".($log::loglevel<6 ? 'q':'')."r \"$srcdir\" rd-qa\@qadb.suse.de:$destdir/");
	&log(LOG_ERR,"Storing the results '$srcdir'=>'$destdir' failed with $ret2") if $ret2;
	return $ret1+$ret2;
}

sub mail # $from, $to, $cc, $subject, $text
{
	my ($from,$to,$cc,$subject,$text)=@_;
	return !system("echo -e \"$text\" | mail -s \"$subject\" -c \"$cc\" -r \"$from\" $to");
}


# Splits a product-release text into its two parts.
# One arg:		e.g. SLES-9-SP3-beta2
# Result value:		2-element list: e.g. "SLES-9-SP3", "beta2"
#
sub set_product_release {
	# That way?
	my $pr_n_r=$_[0];
	my $lasthyphen=rindex $pr_n_r,"-";
	return ((substr $pr_n_r,0,$lasthyphen),(substr $pr_n_r,$lasthyphen+1));
}


# Removes key/val pairs having duplicite keys
#
# Arguments:
#	@_: (key1,val1,key2,val2....)
# Return value: subset of @_ with unique keys
#
sub remove_duplicite_keyvals
{
    my %oldkeys=();
    my @ret=();
    my ($key,$val);
    while( ($key,$val) = splice(@_,0,2) )
    {
        next if $oldkeys{$key};
        push @ret,$key,$val;
        $oldkeys{$key}=1;
    }
    return @ret;
}

# returns (testsuite, testdate)
sub process_dirname # basename of the dir
{
	my ($testsuite,$testdate);
	$testsuite=$_[0];
	$testsuite =~ s/-(2\d\d\d-\d\d-\d\d-\d\d-\d\d-\d\d)$//;
	if( $1 )
	{
		$testdate=$1;
		$testdate = ( $testdate =~ /([12]\d{3})-(\d{2})-(\d{2})-(\d{2})-(\d{2})-(\d{2})/ ? "$1-$2-$3 $4:$5:$6" : undef ) if $testdate
	}
	return ($testsuite,$testdate);
}

1;

