#!/usr/bin/perl -w
# autoyast XML patcher
# currenlty used for merging results from reinstall script with a template
# can also clone system via autoyast and merge with the result

use XML::Simple;
use Getopt::Std;
use Sys::Hostname;

sub HELP_MESSAGE
{
    print <<EOF;
$0 [options] <infile> -m <modfile> <outfile>
$0 [options] -c -m <modfile> <outfile>
options:
\t-c\t\tclone the system, use the result as <infile>
EOF
    exit;
}

sub VERSION_MESSAGE
{
    print "$0 version 1.1\n";
}

$Getopt::Std::STANDARD_HELP_VERSION=1;

getopts('hcq:p:t:v:s:m:',\%opts);
#$qainst_dir=$opts{'q'};
#$patterns=$opts{'p'};
#$packages='qa_tools,hamsta,autoyast2,vim,mc,nfs-client,yast2-nfs-client,iputils,less,screen,lsof,pciutils,lynx,tcpdump,telnet,zip,yast2-runlevel,SuSEfirewall2';
#$packages=$opts{'q'};
$file_mod=$opts{'m'};
if( $opts{'c'} )
{   
    &command( 'yast /usr/share/YaST2/clients/clone_system.ycp' );
    $file_in ='/root/autoinst.xml';
    $file_out=$ARGV[0];
}
else
{
    $file_in =$ARGV[0];
    $file_out=$ARGV[1];
}

# check the arguments, read the input XML
&HELP_MESSAGE() unless $file_in and $file_out;
die "Cannot read input  file '$file_in': $!"  unless -r $file_in;

my $in = XMLin($file_in, ForceArray=>1, ForceContent=>1);

my $mod = undef;
if( $file_mod )
{
    die "Cannot read mod file '$file_mod': $!" unless -r $file_mod;
    $mod = XMLin($file_mod, ForceArray=>1, ForceContent=>1);
}

#&remove($ref,'profile','bootloader','sections','section','vgamode');
#&add($ref,{'vgamode'=>[ {'content'=>'extended'} ] },'profile','bootloader','sections','section');
#&replace($ref,{'content'=>'extended'},'profile','bootloader','sections','section','vgamode');

#$ref2=&dup($ref);

#$tags_intersect = 
#    {
#        'firewall' =>
#        {
#            'enable_firewall' => undef,
#            'start_firewall' => undef
#        },
#        'general' => undef,
#        'keyboard' => undef,
#        'language' => undef,
#        'ldap' => undef,
#        'networking' => undef,
#        'nis' => undef,
#        'printer' => undef,
#        'proxy' => undef,
#        (!defined $patterns ?
#            ('software' =>
#            {
#                'patterns' => undef
#            }) : ()),
#        'timezone' => undef,
#        'user_defaults' => undef,
#    }
#;

#$tags_remove = [ [ 'profile', 'general', 'mouse' ] ];

#$ref = &intersect($ref, $tags_intersect );
#foreach my $tag( @$tags_remove )
#{   &remove( $ref, @$tag ); }
$ref = &merge( $in, $mod ) if $mod;
$ref->{'xmlns'}="http://www.suse.com/1.0/yast2ns";
$ref->{'xmlns:config'}="http://www.suse.com/1.0/configns";

open $file, ">$file_out" or die "Cannot open $file_out for writing: $!";
print $file '<?xml version="1.0"?>'."\n<!DOCTYPE profile>\n";
XMLout( $ref, OutputFile=>$file, RootName=>'profile' );
close $file;
$host = hostname;
$cmd="sed -i 's/HOSTNAME/$host/' $file_out";
system($cmd);

#$a = &reduce($ref);
#print "error\n" unless $a;



# remove( $xml, @path )
# removes a part of $xml
sub remove
{
    my ($root,$key)=splice(@_,0,2);
    return unless $root;
    unless( @_ )
    {
        delete $root->{$key};
        return;
    }
    my $data=$root->{$key};
    return unless $data;
    foreach my $node( @$data )
    {   &remove($node,@_); }
}

# add( $xml, $add, @path )
# adds the $add to every element matching the @path
sub add
{
    my ($root,$add,$key)=splice(@_,0,3);
    return unless $root;
    my $data=$root->{$key};
    return unless $data;
    foreach my $i( 0 .. (@$data-1) )
    {   
        if( @_ )
        {   &add($data->[$i],$add,@_);    }
        else
        {
            $data->[$i] = &merge($data->[$i],$add);
        }
    }
}

# replace( $xml, $repl, @path )
sub replace
{
    my ($root,$repl,$key)=splice(@_,0,3);
    return unless $root;
    my $data=$root->{$key};
    return unless $data;
    foreach my $i( 0 .. (@$data-1) )
    {
        if( @_ )
        {   &replace($data->[$i],$repl,@_); }
        else
        {
            $data->[$i] = $repl;
        }
    }
}

# merge( $xml1, $xml2 )
# merges $xml1 and $xml2 to a new XML tree
sub merge
{
    my ($xml1,$xml2)=@_;
    my $xml={};
    return $xml1 unless $xml2;
    foreach my $key (keys %{$xml1})
    {   @{$xml->{$key}} = map {&dup($_)} @{$xml1->{$key}};  }
    foreach my $key (keys %{$xml2})
    {
        $xml->{$key}=[] unless $xml->{$key};
        if( @{$xml->{$key}}==1 and @{$xml2->{$key}}==1 )
        {   $xml->{$key}->[0] = &merge($xml->{$key}->[0], $xml2->{$key}->[0] ); }
        else
        {   push @{$xml->{$key}}, map {&dup($_)} @{$xml2->{$key}};  }
    }
    return $xml;
}

# dup( $xml )
# duplicates XML tree
sub dup
{
    my $xml=$_[0];
    my $ret={};
    return $xml unless ref($xml) eq 'HASH';
    foreach my $key( keys %{$xml} )
    {   
        if( ref($xml->{$key}) eq 'ARRAY' )
        {   @{$ret->{$key}}=map { &dup($_) } @{$xml->{$key}};    }
        elsif( ! ref($xml->{$key}) )
        {   $ret->{$key} = $xml->{$key};  }
    }
    return $ret;
}

# asymmetrical intersection
# intersect( $xml1, reduce($xml2) )
sub intersect
{
    my ($xml,$mask)=splice(@_,0,2);
    my $ret={};
    return undef unless $xml;
    return $xml unless $mask;
    foreach my $node( keys %$mask )
    {
        next unless $xml->{$node};
        @{$ret->{$node}} = map {&intersect($_,$mask->{$node})} @{$xml->{$node}};
    }
    return $ret;
}

# $ret = reduce($xml)
sub reduce
{
    my $xml=$_[0];
    my $ret={};
    foreach my $key (keys %$xml)
    {   $ret->{$key} = &merge_array(@{$xml->{$key}});   }
    return $ret;
}

# $ret = merge_array(@nodes)
sub merge_array
{
    my $ret={};
    foreach my $subtree (@_)
    {
        foreach my $node (keys %$subtree)
        {
            $ret->{$node}=[] unless $ret->{$node};
            push @{$ret->{$node}}, @{$subtree->{$node}};
        }
    }
    foreach my $node( keys %$ret )
    {   $ret->{$node} = &merge_array(@{$ret->{$node}});    }
    return undef unless keys %$ret;
    return $ret;
}

sub command
{
    my $cmd=$_[0];
    print $cmd,"\n";
    my $ret = system $cmd;
    die "Command failed with code $ret" if $ret>0;
}

