#!/usr/bin/perl -w

BEGIN {
# extend the include path to get our modules found
	push @INC,"/usr/share/qa/lib",'.';
}

use detect;
use Getopt::Std;

$Getopt::Std::STANDARD_HELP_VERSION=1;
getopts('p:s:o:u:r:t:f:b:',\%opts);
if (defined $opts{'p'} and length $opts{'p'} > 3)
     { $source = $opts{'p'}; }
else { die "Usage: ./reinstall.pl -p <Install repo>\n" .
           "                    [ -f <fs type for root> ]\n" .
           "                    [ -s <SDK repo> ]\n" .
           "                    [ -o <Install options> ]\n" .
           "                    [ -u <User profile> ]\n" .
           "                    [ -r <Additional RPMS> ]\n" .
           "                    [ -t <Additional patterns> ]\n" .
	   "                    [ -b <MBR|root> (where to install bootloader) ]\n"; }
$url_sdk = $opts{'s'};
$installoptions = $opts{'o'};
$userprofile = $opts{'u'};
$additionalrpms = $opts{'r'};
$additionalpatterns = $opts{'t'};
$rootfstype = $opts{'f'} ? $opts{'f'} : 'ext3';
$defaultboot = $opts{'b'} ? $opts{'b'} : '';
$tooldir = '/usr/share/qa/tools';
$profiledir = '/usr/share/qa/profiles';
$mountpoint = '/mnt';
$install_tests = 0;
$ENV{'LC_ALL'}='en_US';

# from_type: opensuse|sles
# from_version: \d+
# from_subversion (i.e. SP number in SLES): \d+
($from_type, $from_version, $from_subversion, $arch) = &parse_suse_release;

if ( $userprofile ){
    $ay_xml = $userprofile;
} else {
# location: cz|de|cn|us
    $location = &get_location or die "Unknown location (Prague|Nuernberg|Beijing|Provo)";
    print "Location: $location\n";

    die "Cannot identify current distro" unless $from_type and $from_version;
    $from_subversion = 0 unless $from_subversion;
    print "Distro:\n", &stats( $from_type, $from_version, $from_subversion, $arch );

# to_type: opensuse|sles|sled|slert|slepos
    ($to_type, $to_version, $to_subversion, $to_arch) = &parse_source_url( $source ) or die "Cannot understand your URL";
    if ( $to_arch =~ /^i[3456]86$/ )
    { $to_arch = 'i386'; $to_arch = 'i586' if $to_version eq '11'; }
    print "Reinstalling to:\n", &stats( $to_type, $to_version, $to_subversion, $to_arch );

    $verstring = "sle".$to_version;
    if ( $to_subversion ne '0' )
    { $verstring .= "-sp".$to_subversion; }

#Location deteted. Define SDK source and autoinstall profile location
    if( $location eq 'cz' or $location eq 'de' or $location eq 'us' )
    { $profile_url_base = 'http://10.20.1.229/autoinst';
        `mount 10.20.1.229:/srv/hamsta $mountpoint -o nolock;`
    }
    elsif( $location eq 'cn' )
    { $profile_url_base = 'http://147.2.207.242/autoinst/';
        `mount 147.2.207.242:/mirror_a $mountpoint -o nolock;`
    }

    $patterns = &get_patterns( $to_type, $to_version, $to_subversion );
    $packages = &get_packages( $to_type, $to_version, $to_subversion );
    $profile = &get_profile( $to_type, $to_version, $to_subversion );
    $modfile = &make_modfile( $source, $url_sdk, $to_type, $to_version, $to_subversion, $patterns, $packages, $defaultboot );
    $ay_xml = &install_profile( $profile, $modfile );
    &command( "umount $mountpoint" );
}
if ($arch eq 'ia64' or $arch eq 'ppc64') { $aytool="setupeliloforinstall"; }
else { $aytool="setupgrubforinstall"; }
my $cmdline="$tooldir/$aytool $source autoyast=$ay_xml install=$source";
$cmdline .= " $installoptions" if defined $installoptions;
&command($cmdline);

&command( "sleep 2" );
&command( "reboot" );
exit -1;

################################################################################

# formats distro info
sub stats
{
    my ($type,$version,$subversion,$arch)=@_;
    my $out="  type\t\t$type\n  version\t$version\n  subversion\t$subversion\n";
    $out .= "  arch\t\t$arch\n" if $arch;
    return $out;
}

# gets distro info from the install URL
sub parse_source_url
{
    my $name=$_[0] or return ();
    my ( $type, $ver, $sub, $arch);

    if( $name =~ /(os|opensuse|suse-linux|suse)-?(\d+)\.(\d+)/i  or
            $name =~ /(full)-(\d+).(\d+)-(\w+)/i)
    {   
        ($type,$ver,$sub) = ( 'opensuse', $2, $3 ); 
    }
    elsif(  $name =~ /(sles?|sled)-?(\d+)(.*?-?sp-?(\d+))?/i or
            $name =~ /full-(sles?)(\d+)(-sp(\d+))?/i )
    {   
        ($type,$ver,$sub) = ( 'sles', $2, $4 );
        $type=lc($1) if lc($1) eq 'sled';
        $type = 'slert' if $name =~ /-(slert|rt)-/i;
        $type = 'slepos' if $name =~ /-slepos-/i;
        $sub = 0 unless $sub;
    }
    if ($name =~ /(i\d86|ppc(64)?|s390x?|ia64|x86[_-]64)/i)
    {
        $arch = lc $1;
        $arch =~ s/-/_/g;
    }
    return ($type, $ver, $sub, $arch);
}

# runs a command
sub command
{
    my $cmd=$_[0];
    print $cmd,"\n";
    my $ret = system $cmd;
    die "Command '$cmd' failed with code $ret" if $ret>0;
}

# patches a file
sub patch_file
{
    my ($file,$patch) = @_;
    print "patching file '$file'\n";
    my $patchfile = "/tmp/patch_$$";
    open my $handle, ">$patchfile" or die "Cannot create temporary file '$patchfile': $!";
    print $handle $patch;
    close $handle;
    &command( "patch \"$file\" \"$patchfile\"" );
}

# installs packages using zypper
sub zypper_install_packages
{
    my @packages = @_;
    &command( 'zypper -n --no-gpg-checks install -l '.join(' ',@packages) );
}

# <C-S-F3>
sub disk_stats
{
    my $partition=$_[0];
    my ($dev,$num);
    if( $partition =~ /^(\/dev\/\w+)(\d+)$/ )
    {   ($dev,$num)=($1,$2);    }
    else
    {   return undef;   }

    if ( ( $dev =~ /dev\/sd/ or $dev =~ /dev\/hd/ ) and `lsmod | grep ide_gd_mod` )
    {
        $short=substr($dev,-1,1);
        if ( $to_version > 10 )
        { $dev="/dev/sd".$short; }
        elsif ( $to_version == 10 and $to_subversion > 2 )
        { $dev="/dev/sd".$short; }
        else
        { $dev="/dev/hd".$short; }
    }
    return ($dev,$num);
}

# reads root and swap partition info
sub read_partitions
{
    my ($new_type,$new_version,$new_subversion)=@_;
    my $swappart=`cat /proc/swaps | tail -n 1 | cut -f1 -d' '`;
    my $rootpart=`df /|tail -n1 | cut -f1 -d' '`;
    my $bootpart;
    if ($arch eq 'ppc64')
    {
        $bootpart="/dev/" . `ls -l \$(grep ^boot /etc/lilo.conf | cut -d \" \" -f3) \| awk -F/ {'print \$NF'}`;
    }
    else
    {
        $bootpart=`df /boot/efi 2>/dev/null | tail -n1 | cut -f1 -d' '`;
    }
    my ($rootid,$rootnum) = &disk_stats($rootpart);
    my ($swapid,$swapnum) = &disk_stats($swappart);
    my ($bootid,$bootnum) = &disk_stats($bootpart);
    return ($rootid,$rootnum,$swapid,$swapnum,$bootid,$bootnum);
# TODO: check for undefined results
}

sub get_patterns
{
    my ($to_type,$to_version,$to_subversion) = @_;
    my $ret = "";
    if($to_type eq 'sled')
    {
        if($to_version > 10)
        {
            $ret = "desktop-base,desktop-gnome";
        }
        else
        {
            $ret = "desktop-gnome";
        }
    }
    if (defined $additionalpatterns)
    {
        if ($ret eq "")
        {
            $ret = $additionalpatterns;
        }
        else
        {
            $ret .= ",$additionalpatterns";
        }
    }
    if ($ret eq "")
    {
        return undef;
    }
    return $ret;
}

sub get_packages
{
    my ($to_type,$to_version,$to_subversion) = @_;
    my $ret='qa_tools,hamsta,autoyast2,vim,mc,iputils,less,screen,lsof,pciutils,tcpdump,telnet,zip,yast2-runlevel,SuSEfirewall2,curl,wget,perl,openssh';
    if( $install_tests )
    {
        $ret .= join '', map {",$_-ctcs2-glue"} qw(bonnie contest dbench interbench kernbench libmicro lmbench ltp netperf reaim specweb sysbench tiobench);
        $ret .= ",autotest-ctcs2-glue,qa_siege";
    }
    if( $to_type eq 'opensuse' )
    {	$ret .= ',nfs-client';	}
    else
    {	$ret .= ',nfs-utils';	}
    if (defined $patterns)
       { $ret .= ",setvncserver" if $patterns =~ /desktop/; }
    if (defined $additionalrpms)
    { $ret .= ",$additionalrpms"; }
    return $ret;
}

sub get_profile
{
    my ($to_type,$to_version,$to_subversion) = @_;
    return "$profiledir/sles9.xml" if $to_version<10;
    return "$profiledir/default.xml";
}

sub get_buildservice_repo
{
# TODO
# SLES_9, SLE_10_SP1_Head, SLE_10_SP2_Head, SLE_Factory, openSUSE_11.0, openSUSE_Factory
    my ($type,$version,$subversion) = @_;
    if( $type eq 'opensuse' )
    {
        return 'openSUSE_11.0' if $version==11 and $subversion==0;
        return 'openSUSE_11.1' if $version==11 and $subversion==1;
        return 'openSUSE_Factory' if $version>=11;
    }
    else
    {
        return 'SLES_9' if $version==9;
        return 'SLE_10_SP1_Head' if $version==10 and $subversion<=1;
        return 'SLE_10_SP2_Head' if $version==10 and $subversion==2;
        return 'SLE_10_SP3' if $version==10;
        return 'SUSE_SLE-11_GA' if $version==11 and $subversion==0;
        return 'SUSE_SLE-11-SP1_GA' if $version==11;
        return 'SLE_Factory' if $version>11;
    }
    return undef;
}

# creates a modification file for AutoYaST
sub make_modfile
{
    my ($source,$url_sdk,$new_type,$new_version,$new_subversion,$patterns,$packages,$defaultboot)=@_;
# put it all into one function,
# would return (undef,undef) if the partition does not exist
    my ($rootid,$rootnum,$swapid,$swapnum,$bootid,$bootnum) =
        &read_partitions($new_type,$new_version,$new_subversion);
    my $bsurl = &get_buildservice_repo($new_type, $new_version, $new_subversion);
    my %urls=();
    $QADev_repo="http://dist.suse.de/ibs/QA:/Head";
    $QA_repo="http://dist.suse.de/ibs/QA:/Head";
    $QA_repo=$QADev_repo if $new_version==11 or ($new_version==10 and $new_subversion==3);
    $urls{'BuildService'}="$QA_repo/$bsurl" if $bsurl;
    $urls{'SDK'}=$url_sdk if defined $url_sdk;

# FIXME WTF??? probably completely useless line!!! @urls not known
    #push @urls,$url_sdk;

    my $modfile="/tmp/modfile_$$.xml";
    open my $f, ">$modfile" or die "Cannot create patch file '$modfile' : $!";

    print $f '<profile xmlns="http://www.suse.com/1.0/yast2ns" xmlns:config="http://www.suse.com/1.0/configns">'."\n";
    print $f " <install>\n" if $new_version<10;
    print $f <<EOF;
    <add-on>
        <add_on_products config:type="list">
EOF
        foreach my $urlkey (keys %urls)
        {
            my $urlval=$urls{$urlkey};
            next if (!$urlval);
            print $f <<EOF;
            <listentry>
                <media_url>$urlval</media_url>
                <product>$urlkey</product>
                <product_dir>/</product_dir>
                <accept_unknown_gpg_key config:type="boolean">true</accept_unknown_gpg_key>
                <accept_unsigned_file config:type="boolean">true</accept_unsigned_file>
                <accept_file_without_checksum config:type="boolean">true</accept_file_without_checksum>
                <accept_verification_failed config:type="boolean">true</accept_verification_failed>
                <accept_non_trusted_gpg_key config:type="boolean">true</accept_non_trusted_gpg_key>
                <import_gpg_key config:type="boolean">true</import_gpg_key>
                <ask_on_error config:type="boolean">false</ask_on_error>
                </listentry>
EOF
        }
    print $f <<EOF;
    </add_on_products>
        </add-on>
EOF

    my $drives={};
    $drives->{$rootid}->{$rootnum}='/';
    $drives->{$swapid}->{$swapnum}='swap' if defined $swapid;
    $drives->{$bootid}->{$bootnum}='/boot/efi' if defined $bootid and $arch eq 'ia64';
    $drives->{$bootid}->{$bootnum}='NULL' if defined $bootid and $arch eq 'ppc64';

    my %fs = ( '/'=>$rootfstype, 'swap'=>'swap', '/boot/efi'=>'vfat', 'NULL' => 'ext3');
    my %format = ( '/'=>'true', 'swap'=>'false', '/boot/efi'=>'true', 'NULL' => 'false' );

    print $f "  <partitioning config:type=\"list\">\n";
    foreach my $drive ( keys %{$drives} )
    {
        print $f "   <drive>\n";
        print $f "    <device>$drive</device>\n";
        print $f "    <use>".(join ',', keys %{$drives->{$drive}} )."</use>\n";
        print $f "    <partitions config:type=\"list\">\n";
        foreach my $num ( keys %{$drives->{$drive}} )
        {
            my $mnt=$drives->{$drive}->{$num};
            print $f "     <partition>\n";
            print $f "      <filesystem config:type=\"symbol\">".$fs{$mnt}."</filesystem>\n";
            print $f "      <mount>$mnt</mount>\n" if $mnt ne 'NULL';
            print $f "      <create config:type=\"boolean\">false</create>\n";
            print $f "      <format config:type=\"boolean\">".$format{$mnt}."</format>\n";
            print $f "      <partition_id config:type=\"integer\">65</partition_id>\n" if $mnt eq 'NULL';
            print $f "      <partition_nr config:type=\"integer\">$num</partition_nr>\n";
            print $f "     </partition>\n";
        }
        print $f "    </partitions>\n";
        print $f "   </drive>\n";
    }
    print $f "  </partitioning>\n";

    my $pat='pattern';
    my $pats='patterns';
# TODO: since when it is patterns and not RPM groups ?
    if( $patterns or $packages )
    {
        print $f "  <software>\n";
        if( $patterns )
        {
            print $f "    <$pats config:type=\"list\">\n";
            print $f map {"      <$pat>".$_."</$pat>\n"} split(/,/,$patterns);
            print $f "    </$pats>\n";
        }
        if( $packages )
        {
            print $f "    <packages  config:type=\"list\">\n";
            print $f map {'      <package>'.$_."</package>\n"} split(/,/,$packages);
            print $f "    </packages>\n";
        }
        print $f "  </software>\n";
    }
    if( $defaultboot ne 'root' )
    {
	print $f "  <bootloader>\n";
	print $f "   <global>\n";
	if( $defaultboot =~ /^MBR$/i )	# bootloader in MBR
	{	print $f "    <boot_mbr>true</boot_mbr>\n";	}
	else	# bootloader in root partition, 
	{	
		print $f "    <activate>false</activate>\n";	
		print $f "    <boot_root>true</boot_root>\n";
		print $f "    <boot_extended>false</boot_extended>\n";
		print $f "    <generic_mbr>false</generic_mbr>\n";
	}
	print $f "   </global>\n";
	print $f "  </bootloader>\n";
    } 
    # if $defaultboot is set to 'root', we do not generate anything here
    # then the default is used - bootloader in root, active flag set

    print $f " </install>\n" if $new_version<10;

    print $f "</profile>\n";
    close $f;
    return $modfile;
}

# modifies the profile and stores the result on a NFS share on bender
# returns URL to the result
sub install_profile
{
    my ($profile,$modfile) = @_;
    my $hostname = `hostname`;
    chomp $hostname;
    my $xml_out = "$mountpoint/autoinst/autoinst_$hostname.xml";
    &command( "$tooldir/modify_xml.pl -t '$to_type' -v '$to_version' -s '$to_subversion' -m '$modfile' '$profile' '$xml_out'" );
    return "$profile_url_base/autoinst_$hostname.xml";
}


