#!/usr/bin/perl -w

BEGIN {
# extend the include path to get our modules found
push @INC,"/usr/share/qa/lib",'.';
}

use strict;
use POSIX qw /strftime/;
use detect;
use log;
my $dir='/var/log/qa';
chomp(my $time=`date '+%F-%H-%M-%S'`);

my $host='';
my $arch = '';
my $product = '';
my $kernel = '';

my $rbase="/tmp";
# TODO Change me before release!
my $rhost="qadb.suse.de";
my $ruser="rd-qa";
my $tcflist = undef;

my $nomove=0;
my $delete=0;
$nomove=1 if (`ls $dir | wc -l` < 1);
my $interactive=1; # AKA no batchmode
my $argf_index;

our $VERSION='0.1';
&check_version('remote_qa_db_report.pl',$VERSION);

sub usage
{
	print
"Usage: $0 [ -p PRODUCT] [-c <comment>] [-b] [-C] [-L] [-D] [-A] [-v <n>] [-a ARCH] [-f PATH] [-F TCF_LIST] [-k KERNEL] [-m TESTHOST] [-t TYPE] [-T TESTER]\n",
"       $0 -h\n",
"\n",
"Options and option values (options may be given in any order):\n",
"\n",
"	-h	print this help and exits\n",
"	-b	batch mode (not interactive at all)\n",
"	-c <comment>	submission comment, max. 100 chars, will be truncated when longer \n",
"	-v n	verbosity level (0-7, 5 is default, 3 only prints warnings+errors )\n",
"	-L	Omits moving the submitted logs from PATH to PATH/oldlogs/\n",
"	-R	Delete submitted logs from PATH (do not move to oldlogs)\n",
"	-D	No writing to the database\n",
"	-A	Do not scp the submitted data to the archive\n",
"	-C	commit after every TCF is inserted\n",
"\n",
"	PRODUCT:	e.g. SLES-10-beta1 | SLES-9-SP4-RC1\n",
"	ARCH:		QADB architecture, e.g. i586,ia64,x86_64,ppc,ppc64,s390x,xen0-*...\n",
"       		(default: detected arch of this host)\n",
"	KERNEL:		kernel version for KOTD tests\n",
"	PATH:		the ctcs2 directory containing results to submit\n",
"       		(default: $dir)\n",
"	TCF_LIST:	comma-separated list of test-runs (ctcs2 subdirs) that should be processed.\n",
"				If not set, all test-runs are processed.\n",
"				Example: -F 'qa_siege-2009-12-03-11-13-37,qa_siege-2009-12-03-12-21-12'\n",
"	TESTHOST:	hostname of system under test\n",
"			(default: hostname of this machine)\n",
"	TYPE:		kotd, patch:<md5sum>, product (default: product)\n",
"	TESTER:		login of the tester (default: hamsta-default)\n" ;
}

# process args
foreach my $i (0 .. @ARGV-1)
{	
	my $a=$ARGV[$i];
	if( $a eq '-f' )
	{	$argf_index=$i+1;	}
	elsif( $a eq '-a' )
	{	$arch=$ARGV[$i+1];	}
	elsif( $a eq '-k' )
	{	$kernel=$ARGV[$i+1];	}
	elsif( $a eq '-m' )
	{	$host=$ARGV[$i+1];	}
	elsif( $a eq '-b' )
	{	$interactive=0;		}
	elsif( $a eq '-L' )
	{	$nomove=1;		}
	elsif( $a eq '-R' )
	{	$delete=1; $nomove=1;	}
	elsif( $a eq '-F' )
	{	$tcflist=$ARGV[$i+1];		}
	elsif( $a eq '-p' )
	{	$product=$ARGV[$i+1];		}
	elsif( $a =~ /^--?(h|help|\?)$/ )
	{	&usage();	exit;	}
}


# Convert /var/log/ctcs2 to new structure - Can delete it in future! TODO
`if [ -d /var/log/ctcs2 -a ! -L /var/log/ctcs2 ] ; then mkdir -p /var/log/qa/ctcs2 ; mv /var/log/ctcs2/* /var/log/qa/ctcs2 ; rmdir /var/log/ctcs2; ln -s qa/ctcs2 /var/log/ctcs2 ; fi`;
# allow specify tcflist without ctcs2/ prefix
$tcflist=join ',', map /\// ? $_ : "ctcs2/".$_, split ',', $tcflist if $tcflist;
# end of TODO

# detect local machine
if ($arch eq '') {
	$arch = &get_architecture();
	push @ARGV,'-a',$arch;
}

if ($host eq '') {
	chomp($host=`hostname`);
	push @ARGV,'-m',$host;
}

if ($product eq '') {
	my @p = &detect_product;
	$product = $p[5].'-'.$p[3];
	push @ARGV,'-p',$product;
}

if( $kernel eq '' )
{	push @ARGV, '-k', &get_kernel_version();	}

my $rfile="$host-$time.tar.bz2";

# process path argument
if($argf_index)
{
	$ARGV[$argf_index]=$dir unless $argf_index<@ARGV;
	$dir=$ARGV[$argf_index];
	
	#$ARGV[$argf_index-1]='-f';
	$ARGV[$argf_index]="$rbase/$rfile";
}
else
{	push @ARGV,'-f',"$rbase/$rfile";	}

unless ($nomove) {
	# There's no need to move - all remote submission has unique dir
	push @ARGV,'-L';
}

unless ($delete) {
	# There's no need to keep copy of logs on submit server
	push @ARGV,'-R';
}
	

#push @ARGV, '-f', `basename $dir`; #We have -X specified -> local path in archive

# pack info - create _REMOTE subdir with such infos
my $metadir="$dir/_REMOTE";
mkdir $metadir or die "Unable to create directory '$metadir' for information transfer: $!\n"
						."It is possible that another submit is running. If you're sure it's not, "
						."delete directory and start again.\n";
&log(LOG_INFO,"Getting RPM info");
system('rpm -qa --qf "%{NAME} %{VERSION}-%{RELEASE}\n" | sort > "'.$metadir.'/rpmlist"');
&log(LOG_INFO,"Getting hwinfo");
system("/usr/sbin/hwinfo --all | grep -v events | grep -v Clock: | grep -v 'Memory Range' > '$metadir/hwinfo'");


#my ($type, $version, $subversion, $ar) = &parse_suse_release();
#print "type:$type version:$version subversion:$subversion arch:$arch\n";
#exit;
#my $data={ 'arch'=>$arch, 'host'=>$host, 'product'=>''}; # TODO: product


my $tmpfile="/tmp/$rfile";
my $base=`basename $dir`;
chdir $dir or die "Cannot chdir to $dir: $!";
if ($tcflist) {
	system "tar cf - ".(join ' ', map {(/ / ? '"'.$_.'"' : $_)} split(/,/, $tcflist))." | bzip2 -f > \"$tmpfile\"" and die "Cannot create archive.";
} else {
	system "ls | grep -v '^oldlogs\$' | xargs tar cf - | bzip2 -f > \"$tmpfile\"" and die "Cannot create archive.";
}
&log(LOG_INFO,"Copying files over network");
system "scp \"$tmpfile\" $ruser\@$rhost:$rbase/" and die "Cannot SCP";
unlink $tmpfile or warn "Cannot unlink $tmpfile : $!";

my $cmd="ssh $ruser\@$rhost /usr/share/qa/tools/qa_db_report.pl '".(join ' ',map {(/ / ? '"'.$_.'"' : $_)} @ARGV)."'";
&log(LOG_INFO,$cmd);
my $qadb_report_res=system ($cmd);

unlink "$metadir/hwinfo";
unlink "$metadir/rpmlist";
rmdir $metadir;

die "Cannot run $cmd" if $qadb_report_res;

# process logs - move to oldlogs/
# TODO: when using inotify, should move just after every submit of TCFs
my $oldlogs="$dir/oldlogs";
my $savedir="$oldlogs/" . strftime ("%F-%H-%M-%S", localtime);
unless( $nomove )
{
	mkdir $oldlogs unless -d $oldlogs;
	mkdir $savedir unless -d $savedir;
	if ( -d $savedir ) {
		if ($tcflist) {
			# format is parser/testsuite
			my $dirlist=join ' ', map {(/ / ? '"'.$_.'"' : $_)} split(/,/, $tcflist);
			`for i in $dirlist ; do [ "\$i" == "oldlogs" ] || ( mkdir -p "$savedir/\$(dirname "\$i")" && mv "\$i" "$savedir/\$(dirname "\$i")" ) ; done`;
		} else {
			# all parsers, no need create subdir, aleready have a right structure
			`for i in * ; do [ "\$i" == "oldlogs" ] || for j in "\$i"/* ; do mkdir -p "$savedir/\$i"; mv "\$j" "$savedir/\$i" ; done ; done`;
		}
	} else {
		&log(LOG_ERR,"Unable to move Logs to target: $savedir. Not moving logs to oldlogs.\n");
	} 
}

if($delete)
{
		if ($tcflist) {
			# format is parser/testsuite
			my $dirlist=join ' ', map {(/ / ? '"'.$_.'"' : $_)} split(/,/, $tcflist);
			`for i in $dirlist ; do rm -fr "\$i"; done`;
		} else {
			# all parsers, no need create subdir, aleready have a right structure
			`for i in * ; do [ "\$i" == "oldlogs" ] || for j in "\$i"/* ; do rm -fr "\$j"; done ; done`;
		}
}

